import hashlib
import base64
from _pysha3 import keccak_256
from ecdsa import SigningKey, SECP256k1

def ecdsa_verifing(signature, data, verifing_key, hashfunc=hashlib.sha256):
    """
    AOS ECDSA 验证签名
    :param verifing_key:
    :param signature:
    :param data:
    :param hashfunc:
    :return:
    """
    verify = verifing_key.verify(signature=signature, data=data, hashfunc=hashfunc)
    return verify

def ecdsa_sign(data, privkey, hashfunc=hashlib.sha256):
    """
    AOS ECDSA 签名
    :param data:
    :param privkey:
    :param hashfunc:
    :return:
    """
    if len(privkey) == 66:
        privkey = privkey[2:]
    elif privkey != 64:
        return 0, 0, 0
    signning_key = SigningKey.from_string(bytes.fromhex(privkey), curve=SECP256k1)
    verifing_key = signning_key.get_verifying_key()
    signature = signning_key.sign(data, hashfunc=hashfunc)
    return signature, data, verifing_key

def base64_decode(base_data):
    """
    AOS Base64解密
    :param base_data:
    :return:
    """
    bytes_data = base64.b64decode(base_data)
    return bytes_data

def base64_encode(bytes_data):
    """
    AOS Base64加密
    :param bytes_data:
    :return:
    """
    base_data = base64.b64encode(bytes_data)
    return bytes.decode(base_data)

def Hash(msg):
    """
    AOS Hash加密
    :return:
    """
    if isinstance(msg, str):
        msg = bytes(msg, encoding="utf8")
    k = keccak_256()
    k.update(msg)
    return k.hexdigest()

def binary_to_list(bin):
    list = []
    for idx, val in enumerate(bin):
        list.append(val)
    return list


def list_to_binary(list):
    bin = b''
    for i in list:
        bin += bytes([i])
    return bin
a = Hash(b'hello')
print(a)


import requests
import logging
LOG = logging.getLogger(__name__)
class Base(object):
    def __init__(self, url):

        self.BASEURL = url

    def get(self, url, params=""):

        response = requests.get("{BASEURL}{url}".format(BASEURL=self.BASEURL, url=url), params=params)
        if response.status_code >= 400:
            LOG.warning('create charging_rule error: %s:%s', response.status_code, response.text)
            return None
        return response.json()

    def post(self, url, data):

        response = requests.post("{BASEURL}{url}".format(BASEURL=self.BASEURL, url=url), json=data)
        if response.status_code >= 400:
            LOG.warning('create charging_rule error: %s:%s', response.status_code, response.text)
            return None
        return response.json()


import requests
import logging
LOG = logging.getLogger(__name__)
class Base(object):
    def __init__(self, host, port):

        self.HOST = host
        self.PORT = port
        if "://" in host:
            if host[-1] == "/":
                host = host[:-1]
            entity = host.split("/")[2].split(":")
            host = entity[0]
            self.HOST = host
            port = int(entity[1])
            self.PORT = port
        if not isinstance(port, int):
            raise TypeError("port must be an instance of int")
        self.BASEURL = "https://{host}:{port}".format(host=host, port=port)

    def get(self, url, params=""):

        response = requests.get("{BASEURL}{url}".format(BASEURL=self.BASEURL, url=url), params=params)
        if response.status_code >= 400:
            LOG.warning('create charging_rule error: %s:%s', response.status_code, response.text)
            return None
        return response.json()

    def post(self, url, data):

        response = requests.post("{BASEURL}{url}".format(BASEURL=self.BASEURL, url=url), json=data)
        if response.status_code >= 400:
            LOG.warning('create charging_rule error: %s:%s', response.status_code, response.text)
            return None
        return response.json()


from .base import Base

class chain(Base):
    def __init__(self, url):
        super(chain, self).__init__(url)

    def get_info(self):
        """
        获取与节点相关的最新信息

        :return:
        """
        return self.get("/v1/chain/get_info")

    def get_block(self, block_num_or_id):
        """
        获取一个块的信息

        :param block_num_or_id: id值
        :return:
        """
        data_dict = {
            "block_num_or_id": block_num_or_id
        }
        return self.post("/v1/chain/get_block", data=data_dict)

    def get_account(self, account_name):
        """
        获取账户的信息

        :param account_name: 账户名
        :return:
        """
        data_dict = {
            "account_name": account_name
        }
        return self.post("/v1/chain/get_account", data=data_dict)

    def get_code(self, account_name):
        """
        获取智能合约代码

        :param account_name: 账户名
        :return:
        """
        data_dict = {
            "account_name": account_name,
            "code_as_wasm": "true"
        }
        return self.post("/v1/chain/get_code", data=data_dict)

    def get_table_rows(self, account_details):
        """
        从帐户中获取智能合同数据。

        :param account_details: (str)帐户详细信息，格式
        must be a string of a json
        :return: response object
        """
        path = '/v1/chain/get_table_rows'

        return self.post(path, data=account_details)

    def abi_json_to_bin(self, data):
        """
        将json序列化为二进制十六进制。生成的二进制十六进制通常是
        用于push_transaction中的数据字段。

        :param data: json 字符串
        :return: response object
        """
        path = '/v1/chain/abi_json_to_bin'
        return self.post(path, data=data)

    def abi_bin_to_json(self, data):
        """
        将二进制十六进制序列化回json。

        :param data: json字符串
        :return:response object
        """
        path = '/v1/chain/abi_bin_to_json'
        return self.post(path, data=data)

    def push_transaction(self, transaction):
        """
        这个方法期望一个JSON格式的事务，并且将会这样做
        试着把它应用到区块链上，

        :param transaction: json字符串
        :return: response object
        """
        path = '/v1/chain/push_transaction'
        return self.post(path, data=transaction)

    def push_transactions(self, transactions):
        """
        此方法一次推送多个事务。

        :param transactions: json字符串
        :return: response object
        """
        path = '/v1/chain/push_transaction'
        return self.post(path, data=transactions)

    def get_required_keys(self, transaction_data):
        """
        从您的密钥列表中获取签署事务所需的密钥。

        :param transaction_data: json字符串
        :return: response object
        """
        path = '/v1/chain/get_required_keys'
        return self.post(path, data=transaction_data)



from setuptools import find_packages, setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='pyaos',
    version='1.1',
    authtor=['kies', 'xiaoyue',],
    packages=find_packages(),
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
)


from .base import Base

class wallet(Base):
    def __init__(self, url):
        super(wallet, self).__init__(url)

    def wallet_create(self, wallet_name):
        """
        使用给定的名称创建一个新的钱包。

        :param wallet_name: (str)要创建的钱包的名称
        :return:
        """
        path = '/v1/wallet/create'
        return self.post(path, data=wallet_name)

    def wallet_open(self, wallet_name):
        """
        打开给定名称的现有钱包。

        :param wallet_name:(str)待打开钱包的名称
        :return:
        """
        path = '/v1/wallet/open'
        return self.post(path, data=wallet_name)

    def wallet_lock_all(self, wallet_name):
        """
        锁上指定名字的钱包

        :param wallet_name:(str)待打开钱包的名称
        :return:
        """
        path = '/v1/wallet/lock_all'
        return self.post(path, data=wallet_name)

    def wallet_lock_all(self):
        """
        锁定所有的钱包。

        :return:
        """
        path = '/v1/wallet/lock_all'
        return self.get(path=path)

    def wallet_unlock(self, wallet_name_password):
        """
        用给定的名称和密码解锁钱包

        :param wallet_name_passord: (str)包含名称和密码的列表
        of the given wallet
        :return:
        """
        path = '/v1/wallet/unlock'
        return self.post(path, data=wallet_name_password)

    def wallet_import_key(self, wallet_name_privKey):
        """
        将私钥导入到给定名称的钱包

        :param wallet_name_privKey:(str)包含钱包的列表
        name and private key
        :return:
        """
        path = '/v1/wallet/import_key'
        return self.post(path, data=wallet_name_privKey)

    def wallet_list(self):
        """
        列出所有的钱包

        :return:
        """
        path = '/v1/wallet/list_wallets'
        return self.get(path)

    def wallet_list_keys(self):
        """
        列出所有钱包上的所有密钥对

        :return:
        """
        path = '/v1/wallet/list_keys'
        return self.get(path)

    def wallet_get_public_keys(self):
        """
        列出所有钱包上的所有公钥

        :return:
        """
        path = '/v1/wallet/get_public_keys'
        return self.get(path)

    def wallet_set_timeout(self, timeout):
        """
        设置钱包自动锁定超时(秒)

        :param timeout
        :return:
        """
        path = '/v1/wallet/set_timeout'
        return self.post(path, data=timeout)

    def wallet_sign_trx(self, transaction_data):
        """
        给定一个事务数组，对事务进行签名
        公钥和链id
        :param transaction_data: (str)事务json和公钥列表
        :return: response key
        """
        path = '/v1/wallet/sign_transaction'
        return self.post(path, data=transaction_data)
